@@ -1,24 +1,23 @@
-use crate::portfolio::error::PortfolioError;
-use crate::portfolio::order::OrderEvent;
+use crate::portfolio::OrderEvent;
 use crate::portfolio::position::Position;
-use crate::strategy::signal::{Decision, SignalStrength};
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
+use crate::strategy::{Decision, SignalStrength};
 
-/// Allocates an appropriate OrderEvent quantity.
+/// Allocates an appropriate [`OrderEvent`] quantity.
 pub trait OrderAllocator {
-    /// Returns an OrderEvent with a calculated order quantity based on the input order,
-    /// SignalStrength and potential existing Position.
+    /// Returns an [`OrderEvent`] with a calculated order quantity based on the input order,
+    /// [`SignalStrength`] and potential existing [`Position`].
     fn allocate_order(
         &self,
-        order: OrderEvent,
+        order: &mut OrderEvent,
         position: Option<&Position>,
         signal_strength: SignalStrength,
-    ) -> Result<OrderEvent, PortfolioError>;
+    );
 }
 
-/// Default allocation manager that implements OrderAllocator. Order size is calculated by using the
-/// default_order_value, symbol close value, and SignalStrength.
-#[derive(Debug, Deserialize)]
+/// Default allocation manager that implements [`OrderAllocator`]. Order size is calculated by
+/// using the default_order_value, symbol close value, and [`SignalStrength`].
+#[derive(Copy, Clone, PartialEq, PartialOrd, Debug, Default, Deserialize, Serialize)]
 pub struct DefaultAllocator {
     pub default_order_value: f64,
 }
@@ -26,10 +25,10 @@ pub struct DefaultAllocator {
 impl OrderAllocator for DefaultAllocator {
     fn allocate_order(
         &self,
-        mut order: OrderEvent,
+        order: &mut OrderEvent,
         position: Option<&Position>,
         signal_strength: SignalStrength,
-    ) -> Result<OrderEvent, PortfolioError> {
+    ) {
         // Calculate exact order_size, then round it to a more appropriate decimal place
         let default_order_size = self.default_order_value / order.market_meta.close;
         let default_order_size = (default_order_size * 10000.0).floor() / 10000.0;
@@ -42,15 +41,14 @@ impl OrderAllocator for DefaultAllocator {
             Decision::Short => order.quantity = -default_order_size * signal_strength as f64,
 
             // Exit
-            _ => order.quantity = 0.0 - position.unwrap().quantity,
+            _ => order.quantity = 0.0 - position.as_ref().unwrap().quantity,
         }
-
-        Ok(order)
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::test_util::{order_event, position};
     use super::*;
 
     #[test]
@@ -59,19 +57,21 @@ mod tests {
             default_order_value: 1000.0,
         };
 
-        let mut input_order = OrderEvent::default();
+        let mut input_order = order_event();
         input_order.decision = Decision::CloseLong;
 
-        let mut input_position = Position::default();
+        let mut input_position = position();
         input_position.quantity = 100.0;
 
         let input_signal_strength = 0.0;
 
-        let actual_result = allocator
-            .allocate_order(input_order, Some(&input_position), input_signal_strength)
-            .unwrap()
-            .quantity;
+        allocator.allocate_order(
+            &mut input_order,
+            Some(&input_position),
+            input_signal_strength,
+        );
 
+        let actual_result = input_order.quantity;
         let expected_result = 0.0 - input_position.quantity;
 
         assert_eq!(actual_result, expected_result)
@@ -83,19 +83,21 @@ mod tests {
             default_order_value: 1000.0,
         };
 
-        let mut input_order = OrderEvent::default();
+        let mut input_order = order_event();
         input_order.decision = Decision::CloseShort;
 
-        let mut input_position = Position::default();
+        let mut input_position = position();
         input_position.quantity = -100.0;
 
         let input_signal_strength = 0.0;
 
-        let actual_result = allocator
-            .allocate_order(input_order, Some(&input_position), input_signal_strength)
-            .unwrap()
-            .quantity;
+        allocator.allocate_order(
+            &mut input_order,
+            Some(&input_position),
+            input_signal_strength,
+        );
 
+        let actual_result = input_order.quantity;
         let expected_result = 0.0 - input_position.quantity;
 
         assert_eq!(actual_result, expected_result)
@@ -109,17 +111,15 @@ mod tests {
         };
 
         let order_close = 10.0;
-        let mut input_order = OrderEvent::default();
+        let mut input_order = order_event();
         input_order.market_meta.close = order_close;
         input_order.decision = Decision::Long;
 
         let input_signal_strength = 1.0;
 
-        let actual_result = allocator
-            .allocate_order(input_order, None, input_signal_strength)
-            .unwrap()
-            .quantity;
+        allocator.allocate_order(&mut input_order, None, input_signal_strength);
 
+        let actual_result = input_order.quantity;
         let expected_result = (default_order_value / order_close) * input_signal_strength as f64;
 
         assert_eq!(actual_result, expected_result)
@@ -133,17 +133,15 @@ mod tests {
         };
 
         let order_close = 226.753403;
-        let mut input_order = OrderEvent::default();
+        let mut input_order = order_event();
         input_order.market_meta.close = order_close;
         input_order.decision = Decision::Long;
 
         let input_signal_strength = 1.0;
 
-        let actual_result = allocator
-            .allocate_order(input_order, None, input_signal_strength)
-            .unwrap()
-            .quantity;
+        allocator.allocate_order(&mut input_order, None, input_signal_strength);
 
+        let actual_result = input_order.quantity;
         let expected_order_size = ((default_order_value / order_close) * 10000.0).floor() / 10000.0;
         let expected_result = expected_order_size * input_signal_strength as f64;
 
@@ -159,17 +157,15 @@ mod tests {
         };
 
         let order_close = 10.0;
-        let mut input_order = OrderEvent::default();
+        let mut input_order = order_event();
         input_order.market_meta.close = order_close;
         input_order.decision = Decision::Short;
 
         let input_signal_strength = 1.0;
 
-        let actual_result = allocator
-            .allocate_order(input_order, None, input_signal_strength)
-            .unwrap()
-            .quantity;
+        allocator.allocate_order(&mut input_order, None, input_signal_strength);
 
+        let actual_result = input_order.quantity;
         let expected_result = -(default_order_value / order_close) * input_signal_strength as f64;
 
         assert_eq!(actual_result, expected_result)
@@ -183,21 +179,19 @@ mod tests {
         };
 
         let order_close = 226.753403;
-        let mut input_order = OrderEvent::default();
+        let mut input_order = order_event();
         input_order.market_meta.close = order_close;
         input_order.decision = Decision::Short;
 
         let input_signal_strength = 1.0;
 
-        let actual_result = allocator
-            .allocate_order(input_order, None, input_signal_strength)
-            .unwrap()
-            .quantity;
+        allocator.allocate_order(&mut input_order, None, input_signal_strength);
 
+        let actual_result = input_order.quantity;
         let expected_order_size = ((default_order_value / order_close) * 10000.0).floor() / 10000.0;
         let expected_result = -expected_order_size * input_signal_strength as f64;
 
         assert_ne!(actual_result, 0.0);
         assert_eq!(actual_result, expected_result)
     }
-}
+}
\ No newline at end of file
